/* 
   Please NOTE:  None of the TACACS code available here comes with any
   warranty or support.
*/

/*
 * The first byte of the regexp internal "program" is actually this magic
 * number; the start node begins in the second byte.
 */
#define	MAGIC	0234
